@@ -137,15 +137,18 @@ class SMW(object):
     :ivar site: the pywikibot site to use for requests
     :ivar prefix: the path prefix for this site e.g. /wiki/
     '''
-    def __init__(self, site=None,prefix="/",showProgress=False):
+    def __init__(self, site=None,prefix="/",showProgress=False, queryDivision=1):
         '''
         Constructor
         Args:
             site: the site to use (optional)
+            queryDivision(int): Defines the number of subintervals the query is divided into (must be greater equal 1)
         '''
         self.site=site
         self.prefix=prefix
         self.showProgress=showProgress
+        self.queryDivision=queryDivision
+        self.QUERY_SPLITUP_ID = "Modification date"
     
     def deserialize(self,rawresult):
         """ deserialize the given rawresult according to 
@@ -212,13 +215,35 @@ class SMW(object):
         else:
             return None
 
+    argumentRegex = staticmethod(lambda arg: r"\| *" + arg + r" *= *\d+(?=\s|\||$)")
+
+    @staticmethod
+    def getOuterMostArgumentValueOfQuery(argument, query):
+        """
+        Extracts the integer value of the given argument from the given query
+        Args:
+            argument(string): Argument that should be extracted
+            query(string): smw query where the given argument is assumed
+        Returns:
+            Returns integer value of the given argument in the given query.
+            If the argument occurs multiple times the last one is returned.
+            If it does not occur return None.
+        """
+        if not argument or not query:
+            return None
+        args = re.compile(SMW.argumentRegex(argument), re.IGNORECASE).findall(query)
+        if not args:
+            return None
+        return re.compile("[0-9]+").search(args[-1]).group()
+
+
 class SMWClient(SMW):
     '''
     Semantic MediaWiki access using mw client library
     '''
     
-    def __init__(self, site=None,prefix="/",showProgress=False):
-        super(SMWClient,self).__init__(site,prefix,showProgress=showProgress) 
+    def __init__(self, site=None,prefix="/",showProgress=False, queryDivision=1):
+        super(SMWClient,self).__init__(site,prefix,showProgress=showProgress, queryDivision=queryDivision)
         pass
     
     def info(self):
@@ -234,6 +259,9 @@ class SMWClient(SMW):
 
         API doc: https://semantic-mediawiki.org/wiki/Ask_API
 
+        The query is devided into multiple subqueries if the results of the query exeed the defined threshold.
+        If this happens the query is executed multiple times to retrieve all results without passing the threshold.
+
         Returns:
             Generator for retrieving all search results, with each answer as a dictionary.
             If the query is invalid, an APIError is raised. A valid query with zero
@@ -251,30 +279,142 @@ class SMWClient(SMW):
         if title is None:
             kwargs['title'] = title
 
-        offset = 0
-        done=False
-        count=0
+        if limit is None:
+            # if limit is not defined (via cmd-line), check if defined in query
+            limit = SMW.getOuterMostArgumentValueOfQuery("limit", query)
+        results = None
+        if self.queryDivision == 1:
+            try:
+                results = self.askForAllResults(query, limit, kwargs)
+            except QueryResultSizeExceedException as e:
+                print(e)
+                results = e.getResults()
+        else:
+            results = self.askPartitionQuery(query, limit, kwargs)
+        for res in results:
+            yield res
+
+    def askPartitionQuery(self, query, limit=None, kwargs={}):
+        """
+        Splits the query into multiple subqueries by determining the 'modification date' interval in which all results
+        lie. This interval is then divided into subintervals. The number of subintervals is defined by the user via
+        commandline. The results of all subqueries are then returned.
+        Args:
+            query(string): the SMW inline query to be send via api
+            limit(string): limit of the query
+        Returns:
+            All results of the given query.
+        """
+        (start, end) = self.getBoundariesOfQuery(query, kwargs)
+        print(f"Start: {start}, End: {end}")
+        numIntervals = self.queryDivision
+        calcIntervalBound = lambda start, n: (start + n * lenSubinterval).replace(microsecond=0)
+        calcLimit = lambda limit, numRes: None if limit is None else limit - numResults
+        done = False
+        numResults = 0
+        results = []
         while not done:
-            count+=1
+            lenSubinterval = (end - start) / numIntervals
+            for n in range(numIntervals):
+                if self.showProgress:
+                    print(f"Query {n+1}/{numIntervals}:")
+                tempLowerBound = calcIntervalBound(start,n)
+                tempUpperBound = calcIntervalBound(start,n+1) if (n+1 < numIntervals) else end
+                queryParam = f"{query}|[[{self.QUERY_SPLITUP_ID}:: >={tempLowerBound.isoformat()}]]|[[{self.QUERY_SPLITUP_ID}:: <={tempUpperBound.isoformat()}]]"
+                try:
+                    tempRes = self.askForAllResults(queryParam, calcLimit(limit, numResults), kwargs)
+                    if tempRes is not None:
+                        for res in tempRes:
+                            results.append(res)
+                            numResults += int(res.get("query").get("meta").get("count"))
+                except QueryResultSizeExceedException as e:
+                    # Too many results for current subinterval n -> print error and return the results upto this point
+                    print(e)
+                    if e.getResults():
+                        for res in e.getResults():
+                            results.append(res)
+                    numResults = 0
+                    break
+            done=True
+        return results
+
+    def getBoundariesOfQuery(self, query, kwargs={}):
+        """
+        Retrieves the time interval, lower and upper bound, based on the modification date in which the results of the
+        given query lie.
+        Args:
+            query(string): the SMW inline query to be send via api
+        Returns:
+            (Datetime, Datetime): Returns the time interval (based on modification date) in which all results of the
+            query lie.
+        """
+        queryparam = f"{query}|?{self.QUERY_SPLITUP_ID}|sort={self.QUERY_SPLITUP_ID}|limit=1"
+        queryparamStart = f"{queryparam}|order=asc"
+        queryparamEnd = f"{queryparam}|order=desc"
+        resultsStart = self.site.raw_api('ask', query=queryparamStart, http_method='GET', **kwargs)
+        resultsEnd = self.site.raw_api('ask', query=queryparamEnd, http_method='GET', **kwargs)
+        self.site.handle_api_result(resultsStart)
+        self.site.handle_api_result(resultsEnd)
+        try:
+            retrieveTimestamp = lambda res: int(next(iter(res.get('query').get('results').items()))[1]
+                                                .get('printouts').get(self.QUERY_SPLITUP_ID)[0].get("timestamp"))
+            start = datetime.utcfromtimestamp(retrieveTimestamp(resultsStart))
+            end = datetime.utcfromtimestamp(retrieveTimestamp(resultsEnd))
+            return (start, end)
+        except Exception as e:
+            print(e)
+            return None, None
+
+
+
+    def askForAllResults(self, query, limit=None, kwargs={}):
+        """
+        Executes the query until all results are received of the given limit is reached.
+        If the SMW results limit is reached before all results are retrieved the QueryResultSizeExceedException is raised.
+        Args:
+            query(string): the SMW inline query to be send via api
+            limit(int): limit for the query results, None (default) for all results
+            kwargs:
+        Returns:
+            query results
+        Raises:
+            QueryResultSizeExceedException: Raised if not all results can be retrieved
+        """
+        endShowProgress = lambda showProgress, c: print("\n" if not c % 80 == 0 else "") if showProgress else None
+        offset = 0
+        done = False
+        count = 0
+        res = []
+        while not done:
+            count += 1
             if self.showProgress:
-                sep="\n" if count%80==0 else "" 
-                print(".",end=sep,flush=True)
-                
-            queryParam=u'{query}|offset={offset}'.format(query=query,offset=offset)
+                sep = "\n" if count % 80 == 0 else ""
+                print(".", end=sep, flush=True)
+
+            queryParam = u'{query}|offset={offset}'.format(query=query, offset=offset)
             if limit is not None:
-                queryParam+="|limit={limit}".format(limit=limit)
+                queryParam += "|limit={limit}".format(limit=limit)
+            # print(f"QueryPram: {queryParam}")   #debug purposes
             results = self.site.raw_api('ask', query=queryParam, http_method='GET', **kwargs)
             self.site.handle_api_result(results)  # raises APIError on error
             continueOffset = results.get('query-continue-offset')
             if continueOffset is None:
-                done=True
+                done = True
             else:
-                if limit is not None and continueOffset>=limit:
-                    done=True
-                if continueOffset<offset:
-                    done=True
-            offset=continueOffset
-            yield results
+                if limit is not None and continueOffset >= limit:
+                    done = True
+                elif not results.get('query').get('results') or continueOffset < offset:
+                    # contine-offset is set but result is empty
+                    endShowProgress(self.showProgress, count)
+                    res.append(results)
+                    raise QueryResultSizeExceedException(result=res)
+                if continueOffset < offset:
+                    done = True
+            offset = continueOffset
+            if results.get('query').get('results'):
+                res.append(results)
+        endShowProgress(self.showProgress, count)
+        return res
     
     def rawquery(self,askQuery,title=None,limit=None):
         '''
@@ -305,6 +445,11 @@ class SMWClient(SMW):
         rawresult=self.rawquery(askQuery, title, limit)
         lod=self.deserialize(rawresult)
         return lod
+
+    def updateProgress(self, count):
+        if self.showProgress:
+            sep = "\n" if count % 80 == 0 else ""
+            print(".", end=sep, flush=True)
     
 class SMWBot(SMW):
     '''
@@ -345,4 +490,17 @@ class SMWBot(SMW):
         '''
         rawresult=self.rawquery(ask,limit=limit)
         result=self.deserialize(rawresult)
-        return result
\ No newline at end of file
+        return result
+
+
+class QueryResultSizeExceedException(BaseException):
+    """Raised if the results of a query can not completely be queried due to SMW result limits."""
+    def __init__(self, result=[], message="Query can not completely be queried due to SMW result limits."):
+        super().__init__(message)
+        self.result=result
+
+    def getResults(self):
+        """ Returns the queried results before the exception was raised """
+        return self.result
+
+
